// C Source File
// Created 3/10/2002; 10:42:35 PM


#include "drawstuff.h"
#include "bitmaps.h"


// screen area for clipping
#if defined(_89)
SCR_RECT screen_area={{0,0,159,99}};
#elif defined(_92plus)
SCR_RECT screen_area={{0,0,239,127}};
#endif


//////////////////////////////////////////////////////////////
//
//  draw_bar: histogram origionally by Alex Astashyn
//  precondition: graymode is assumed to be on.
//
void draw_bar(int x,
              int y,
              int length,
              int width,
              int value)
{
	SCR_RECT rect;

	SetPlane(1);
	rect.xy.x0 = x;
	rect.xy.y0 = y;
	rect.xy.x1 = x + width;
	rect.xy.y1 = y + length;

	ScrRectFill(&rect, &screen_area, A_REVERSE);

	if (value != 0)
	{
		rect.xy.y0 = y + length - value + 1;
		ScrRectFill(&rect, &screen_area, A_NORMAL);
	}
}


//////////////////////////////////////////////////////////////
//
//  draw_empty_block_hist: histogram origionally by Alex Astashyn
//  precondition: graymode is assumed to be on.
//
void draw_empty_block_hist()
{
	int i;
	SCR_RECT rect;

#if defined(_89)
	int directions[11][4] = {
		{7,2,7,0},
		{2,-2,2,-2},
		{7,0,7,0},
		{2,2,2,2},
    {7,-1,11,-1},
		{11,-1,5,1},
		{2,2,4,0},
		{7,0,7,0},
		{4,-2,2,0},
		{5,2,9,0},
		{2,-2,-2,-2}};
#elif defined(_92plus)
	int directions[11][4] = {
		{15,2,15,0},
		{2,-2,2,-2},
		{15,0,15,0},
		{2,2,2,2},
    {15,-1,19,-1},
		{19,-1,13,1},
		{2,2,4,0},
		{15,0,15,0},
		{4,-2,2,0},
		{13,2,17,0},
		{2,-2,-2,-2}};
#endif

	SetPlane(0);
	rect.xy.y0 = Q89_92(41,51);
	rect.xy.y1 = Q89_92(50,67);
	
	for (i = 0; i < 7; i++)
	{
		rect.xy.x0 = Q89_92(95,121) + i * Q89_92(9,17);
		rect.xy.x1 = Q89_92(98,128) + i * Q89_92(9,17);
		ScrRectFill(&rect, &screen_area, A_NORMAL);
	}

	SetPlane(1);
	rect.xy.x0 = Q89_92(95,123);
	rect.xy.y0 = Q89_92(52,69);
	rect.xy.x1 = Q89_92(99,126);
	rect.xy.y1 = Q89_92(55,72);
	ScrRectFill(&rect, &screen_area, A_NORMAL);
	
	for (i = 0; i < 11; i++)
	{
		rect.xy.x0 += directions[i][0];
		rect.xy.y0 += directions[i][1];
		rect.xy.x1 += directions[i][2];
		rect.xy.y1 += directions[i][3];
		ScrRectFill(&rect, &screen_area, A_NORMAL);
	}
}


//////////////////////////////////////////////////////////////
//
//  draw_window: gui windows, yoy!
//
void draw_window(int left,
                 int top,
                 int right,
                 int bottom,
                 char *title,
                 int border)
{
  int i, plane;

  #if defined(_89)
  FontSetSys(F_4x6);
  #elif defined(_92plus)
  FontSetSys(F_6x8);
  #endif

  for (plane = 0; plane < 2; plane++)
  {
    SetPlane(plane);

    // draw the title bar
    for (i = 0; i < Q89_92(8,10); i++)
    {
    	DrawLine(left, top+i, right, top+i, A_NORMAL);
    }

    // draw the title screen
    DrawStr(left+2, top+2, title, A_REVERSE);

    // erase the rest of the window
    for (i = Q89_92(8,10); i < bottom-top-1; i++)
    {
    	DrawLine(left+2, top+i, right-2, top+i, A_REVERSE);
    }

    // draw the border
    DrawClipRect(MakeWinRect(left,top,right,bottom),&screen_area,A_REVERSE);
    DrawClipRect(MakeWinRect(left+1,top+1,right-1,bottom-1),&screen_area,A_NORMAL);

  }

  // more border
  if (border == TRUE)
  {
    DrawClipRect(MakeWinRect(left+2,top+Q89_92(8,10),right-2,bottom-2),&screen_area,A_NORMAL);
    SetPlane(0);
    DrawClipRect(MakeWinRect(left+3,top+Q89_92(9,11),right-3,bottom-3),&screen_area,A_NORMAL);
  }
}


//////////////////////////////////////////////////////////////
//
//  draw_box
//
void draw_box(int left,
              int top,
              int right,
              int bottom,
              int fill)
{
  int i;
  SetPlane(1);
  DrawClipRect(MakeWinRect(left,top,right,bottom),&screen_area,A_NORMAL);

  SetPlane(0);
  for (i = top; fill && i <= bottom; i++)
  {
  	DrawLine(left,i,right,i,A_NORMAL);
  }
}


//////////////////////////////////////////////////////////////
//
//  draw_screen: the screen is always in the background
//
void draw_screen(int style)
{
  int i, plane;
  BITMAP *p0, *p1;

  // clear the screen
  memset(GetPlane(0),0,LCD_SIZE);
  memset(GetPlane(1),0,LCD_SIZE);

  // draw windows
  draw_window(0,0,Q89_92(27,39),Q89_92(36,48)*style + Q89_92(34,46),"Lines",0);
  draw_window(Q89_92(88,112),0,Q89_92(160,239),Q89_92(32,40),"Queue",0);
  draw_window(Q89_92(88,112),Q89_92(32,40),Q89_92(160,239),Q89_92(58,75),"Block Statistics",0);
  draw_window(Q89_92(88,112),Q89_92(58,75),Q89_92(160,239),Q89_92(100,127),"Score",0);

  // choose the correct bitmaps
  if(style == TRUE)
  {
    p0 = &quadra0;
    p1 = &quadra1;
  }
  else
  {
    p0 = &tetris0;
    p1 = &tetris1;
  }

  // draw more stuff
  for (plane = 0; plane < 2; plane++)
  {
    SetPlane(plane);

    // draw the correct bitmaps
    BitmapPut(Q89_92(0,6),Q89_92(35,60) + 33*style,plane?p1:p0, &screen_area, A_NORMAL);

    // lines
    for (i = Q89_92(28,40); i < Q89_92(33,46); i++)
    {
      DrawLine(i,0,i,Q89_92(99,127),A_NORMAL);
      DrawLine(Q89_92(55,66)+i,0,Q89_92(55,66)+i,Q89_92(99,127),A_NORMAL);
    }

    // extra lines at bottom of 92
    #if defined(_92plus)
    for (i = 0; i < 3; i++)
    {
    	DrawLine(40,125+i,106,125+i,A_NORMAL);
    }
    #endif

    #if defined(_89)
    FontSetSys(F_6x8);
    #elif defined(_92plus)
    FontSetSys(F_8x10);
    #endif

    DrawStr(Q89_92(91,115), Q89_92(69,90), "Lines:", A_NORMAL);
    DrawStr(Q89_92(91,115), Q89_92(79,102), "Score:", A_NORMAL);

    draw_line_total_numbers(style, 0);
  }

  // histogram
  draw_empty_block_hist();
}


//////////////////////////////////////////////////////////////
//
//  draw_line_total_numbers: try to get 10+
//
void draw_line_total_numbers(int style,
                             int new_lines)
{
  // A_NORMAL = 1
  // A_REVERSE = 0
  // A_REPLACE = 4 

  int i = new_lines;
  char c[4];
  
  // line total numbers
  #if defined(_89)
  FontSetSys(F_4x6);
  #elif defined(_92plus)
  FontSetSys(F_6x8);
  #endif

  for (i = 0; i < 4 + 5*style; i++)
  {
    sprintf(c, "0%d:", i+1);
    DrawStr(3, Q89_92(9,12) + Q89_92(6,8)*i, c, A_REPLACE - 4*(new_lines == i+1));
  }

  if (style == TRUE)
  {
    sprintf(c, "%d ", (new_lines > 9) ? new_lines : 10);
    DrawStr(3, Q89_92(9,12) + Q89_92(6,8)*i, c, A_REPLACE - 4*(new_lines > 9));

    // draw the +
    DrawLine(Q89_92(11,16), Q89_92(11,15) + i * Q89_92(6,8), Q89_92(13,18), Q89_92(11,15) + i * Q89_92(6,8), A_NORMAL);
    DrawLine(Q89_92(12,17), Q89_92(10,14) + i * Q89_92(6,8), Q89_92(12,17), Q89_92(12,16) + i * Q89_92(6,8), A_NORMAL);
  }
}


//////////////////////////////////////////////////////////////
//
//  draw_menu: then the menu items
//
void draw_menu(int row)
{
  // different titles for multiplayer recognition
  char *titles[3] = {"Main Menu", "Main Menu (first calc)", "Main Menu (second calc)"};
  draw_window(Q89_92(2,42),Q89_92(51,69),Q89_92(158,198),Q89_92(77,97),titles[getMachine()],1);
  menu_items(row);
}


//////////////////////////////////////////////////////////////
//
//  menu_items: home menu
//
void menu_items(int highlight)
{
  int i;

  FontSetSys(F_6x8);

  // draw words
  SetPlane(1);
  for (i = Q89_92(62,82); i < Q89_92(73,93); i++)
  {
    DrawLine(Q89_92(10,50),i,Q89_92(150,190),i,A_REVERSE);
  }

  DrawStr(Q89_92(30,70),Q89_92(64,84),"Play",A_NORMAL);
  DrawStr(Q89_92(100,140),Q89_92(64,84),"Options",A_NORMAL);

  // erase old highlight
  SetPlane(0);
  for (i = Q89_92(62,82); i < Q89_92(73,93); i++)
  {
    DrawLine(Q89_92(10,50),i,Q89_92(150,190),i,A_REVERSE);
  }

  // draw box
  draw_box(Q89_92(10,50)+80*highlight,Q89_92(62,82),Q89_92(70,110)+80*highlight,Q89_92(72,92),1);
}


//////////////////////////////////////////////////////////////
//
//  FlashLine
//
void FlashLine(int line, int fill)
{
  int i;
  int plane;

  for (plane = 0; plane < 2; plane++)
  {
    SetPlane(plane);
    for (i = Q89_92(21,20); i < 26; i++)
    {
      DrawLine(Q89_92(33,46),Q89_92(5,6)*(line-25)-i,Q89_92(82,105),Q89_92(5,6)*(line-25)-i,fill);
    }
  }
}


//////////////////////////////////////////////////////////////
//
//  EraseRect
//
void EraseRect(SCR_RECT rect)
{
  int i;
  for (i = 0; i < 2; i++)
  {
    SetPlane(i);
    ScrRectFill(&rect, &screen_area, A_REVERSE);
  }
}


//////////////////////////////////////////////////////////////
//
//  MoveRect
//
void MoveRect(SCR_RECT rect, int distance)
{
  int i;
  for (i = 0; i < 2; i++)
  {
    SetPlane(i);
    ScrRectScroll(&rect, &screen_area, distance * Q89_92(5,6), A_REVERSE);
  }
}


//////////////////////////////////////////////////////////////
//
//  DoMultiplayer
//
BOOL DoMultiplayer(int *sent_lines)
{
  char buffer[2];
  int i = 0;
  int new_lines = 0;
  int height = 0;
  int opp_height = 0;
  BOOL keep_going = TRUE;
  
  buffer[0] = 0;

  OSReadLinkBlock(buffer,1);
  if (buffer[0] > 0)
  {
    if (buffer[0] == 42)
    {
      // you win
      return FALSE;          	
    }
    else if ((buffer[0] > 50) && (buffer[0] < 70))
    {
      new_lines = (int)buffer[0] - 50;
    }
    else if ((buffer[0] > 100) && (buffer[0] < 125))
    {
      height = (int)buffer[0];
    }
  }

  SetPlane(1);
  // if we got some new lines
  if (new_lines != 0)
  {
    // holes for multiplayer
    for (i = Q89_92(19,20) - *sent_lines; i > Q89_92(19,20) - *sent_lines - new_lines; i--)
    {
      DrawClipRect(MakeWinRect(Q89_92(84,107),Q89_92(5,6)*i+1,Q89_92(86,110),Q89_92(5,6)*i+Q89_92(3,4)),&screen_area,A_REVERSE|B_DOUBLE);
    }
    *sent_lines += new_lines;
  }

  if ((height != 0) && (opp_height != height - 100))
  {
    opp_height = height - 100;

    // redo holes on side for multiplayer
    for (i = Q89_92(28,40); i < Q89_92(33,46); i++)
    {
      DrawLine(i,0,i,Q89_92(99,127),A_NORMAL);
    }

    // height of opponent
    for (i = Q89_92(19,20); i > Q89_92(19,20) - opp_height; i--)
    {
      DrawClipRect(MakeWinRect(Q89_92(29,41),Q89_92(5,6)*i+1,Q89_92(31,44),Q89_92(5,6)*i+Q89_92(3,4)),&screen_area,A_REVERSE|B_DOUBLE);
    }
  }
  
  return keep_going;
}

