//========================================================
//        File: drawstuff.c
//
// Description: Functions to draw common components and
//              the entire GUI screen of the program.
//
//          By: Kurt Hoffmann
//        Date: 3/3/2002
//     Version: 1.0
//========================================================


#include "keys.h"
#include <compat.h>
#include <graph.h>


//========================================================
//    Function: DrawTitle
//
// Description: Draws the words and line at the top.
//
//  Parameters: None
//
//     Returns: None
//========================================================
void DrawTitle()
{
  ClrScr();
  FontSetSys(F_8x10);
  DrawStr(15,1,"Key Configurator",A_NORMAL);
  DrawLine(0,12,159,12,A_NORMAL);
  FontSetSys(F_6x8);
}


//========================================================
//    Function: DrawBox
//
// Description: Draws a box around a key name in the list.
//
//  Parameters: int   - draw the box around this line
//              short - attributes (A_NORMAL or A_REVERSE)
//
//     Returns: None
//========================================================
void DrawBox(int line, short attr)
{
  WIN_RECT rect = {108,11 * (line+2) - 7,150,11 * (line+3) - 7};
  DrawClipRect(&rect,ScrRect,attr);
}


//========================================================
//    Function: DrawDisplay
//
// Description: Draws the entire GUI display.
//
//  Parameters: int            - number of commands
//              int            - start displaying this line
//              char**         - list of commands
//              unsigned short - command key rows
//              unsigned short - command key columns
//
//     Returns: None
//========================================================
void DrawDisplay(int count,
                 int start_line,
                 char **key_list,
                 unsigned short *row_list,
                 unsigned short *col_list)
{
	int loop = 0;

  DrawTitle();

  for (loop = 0; loop < count && loop < Q89_92(7,9); loop++)
  {
    const char *key_name = KeyMaskToStr(row_list[start_line + loop], col_list[start_line + loop]);
    DrawStr(0,11 * (loop+2)-5,key_list[start_line + loop],A_NORMAL);
    DrawStr(110,11 * (loop+2)-5,key_name,A_NORMAL);
  }
}


//========================================================
//    Function: FillBox
//
// Description: Fills in the box containing a key name.
//
//  Parameters: int   - the line number of the box to fill
//              short - attribute (A_NORMAL or A_REVERSE)
//
//     Returns: None
//========================================================
void FillBox(int line,
             short attr)
{
  WIN_RECT lower = {109,11 * (line+3) - 8,149,11 * (line+3) - 8};
  WIN_RECT upper = {109,11 * (line+2) - 6,149,11 * (line+2) - 6};
  FillLines2(&lower,&upper,ScrRect,attr);
}


//========================================================
//    Function: SaveFile
//
// Description: Prompt the user if they want to save the
//              file or not.
//
//  Parameters: None
//
//     Returns: BOOL - did they choose yes?
//========================================================
BOOL SaveFile()
{
	BOOL rval = FALSE;
  short input = 0;

  DrawTitle();

  DrawStr(30, 35, "Save Configuration",A_NORMAL);
  DrawStr(65, 52, "Yes",A_NORMAL);
  DrawStr(65, 67, "No",A_NORMAL);
  WIN_RECT yes = {60, 50, 90, 60};
  WIN_RECT no  = {60, 65, 90, 75};
  DrawClipRect(&no,ScrRect,A_NORMAL);

  while (input != KEY_ENTER)
  {
    input = ngetchx();
    if (input == KEY_DOWN)
    {
      DrawClipRect(&yes,ScrRect,A_REVERSE);
      DrawClipRect(&no,ScrRect,A_NORMAL);
    	rval = FALSE;
    }
    else if (input == KEY_UP)
    {
      DrawClipRect(&yes,ScrRect,A_NORMAL);
      DrawClipRect(&no,ScrRect,A_REVERSE);
    	rval = TRUE;
    }
  }

  return rval;
}
