//========================================================
//        File: keyconf.c
//
// Description: Key Configurator is used to edit key
//              configuration files on the TI-89 and
//              TI-92 plus.
//
//          By: Kurt Hoffmann
//        Date: 3/3/2002
//     Version: 1.0
//========================================================


// Save the screen
#define SAVE_SCREEN
#define KEY_MAX 20

#include "fileio.h"
#include "keys.h"
#include "drawstuff.h"
#include <compat.h>
#include <graph.h>

// can be compiled for both calcs
// this variable is defined in keys.h
#if defined(_89)
int _ti89;
#elif defined(_92plus)
int _ti92plus;
#else
#Please define either _89 or _92plus
#endif


//========================================================
//    Function: _main
//
// Description: The first function called in the program.
//              It does all the work.
//
//  Parameters: None
//
//     Returns: None
//========================================================
void _main(void)
{
  ESI argptr;
  InitArgPtr(argptr);
  char *filename = NULL;
  char *key_list[KEY_MAX];
  unsigned short row_list[KEY_MAX];
  unsigned short col_list[KEY_MAX];
  int count = 0;
  int edit_line = 0;
  int start_line = 0;
  short input = 0;

  DrawTitle();

  FontSetSys(F_6x8);

  // get the first argument
  if (GetArgType(argptr) != STR_TAG)
  {
    DrawStr(0,30,"usage:",A_NORMAL);
    DrawStr(10,40,"keyconf(\"filename\")",A_NORMAL);
    DrawStr(10,80,"Press any key to exit.",A_NORMAL);
    ngetchx();
    return;
  }

  filename = GetStrnArg(argptr);

  // open the file
  if ((count = OpenFile(filename, key_list, row_list, col_list)) == -1)
  {
    DrawStr(0,30,"Error opening file:",A_NORMAL);
    DrawStr(10,40,filename,A_NORMAL);
    DrawStr(10,80,"Press any key to exit.",A_NORMAL);
    ngetchx();
    return;
  }

  DrawDisplay(count, start_line, key_list, row_list, col_list);
  DrawBox(edit_line, A_NORMAL);

  while (input != KEY_ESC)
  {
    input = ngetchx();
    if (input == KEY_DOWN && edit_line < count-1 && edit_line < Q89_92(6,8))
    {
      DrawBox(edit_line, A_REVERSE);
    	edit_line++;
    	DrawBox(edit_line, A_NORMAL);
    }
    else if (input == KEY_DOWN && edit_line == Q89_92(6,8) && count > Q89_92(7,9) + start_line)
    {
    	start_line++;
  	  DrawDisplay(count, start_line, key_list, row_list, col_list);
    	DrawBox(edit_line, A_NORMAL);
    }
    else if (input == KEY_UP && edit_line > 0)
    {
      DrawBox(edit_line, A_REVERSE);
    	edit_line--;
    	DrawBox(edit_line, A_NORMAL);
    }
    else if (input == KEY_UP && edit_line == 0 && start_line > 0)
    {
    	start_line--;
  	  DrawDisplay(count, start_line, key_list, row_list, col_list);
    	DrawBox(edit_line, A_NORMAL);
    }
    else if (input == KEY_ENTER)
    {
    	// get new key
    	FillBox(edit_line, A_REVERSE);
    	GetKey(&row_list[edit_line+start_line], &col_list[edit_line+start_line]);
    	DrawDisplay(count, start_line, key_list, row_list, col_list);
      DrawBox(edit_line, A_NORMAL);
    }
  }
  
  // save file at the end
  if (SaveFile() == TRUE)
  {
    WriteFile(filename,count,key_list,row_list,col_list);
  }
}
