//========================================================
//        File: keys.c
//
// Description: Contains the array of key descriptions and
//              a function to translate between numbers
//              and the matching key.
//
//          By: Kurt Hoffmann
//        Date: 3/3/2002
//     Version: 1.0
//========================================================


#include "keys.h"
#include "system.h"


#ifdef _89
const char* key_names[] = {
"alpha","Diamnd","SHIFT","2nd","Right","Down","Left","Up",
"F5",   "CLEAR", "^",    "/",  "*",    "-",   "+",   "ENTER",
"F4",   "BckSpc","T",    ",",  "9",    "6",   "3",   "(-)",
"F3",   "CATLG", "Z",    ")",  "8",    "5",   "2",   ".",
"F2",   "MODE",  "Y",    "(",  "7",    "4",   "1",   "0",
"F1",   "HOME",  "X",    "=",  "|",    "EE",  "STO", "APPS",
"",     "",      "",     "",   "",     "",    "",    "ESC"
};
#else
const char* key_names[] = {
"Down","Right", "Up",   "Left","Hand","Shift","Diamnd","2nd",
"3",   "2",     "1",    "F8",  "W",   "S",    "Z",     "",
"6",   "5",     "4",    "F3",  "E",   "D",    "X",     "",
"9",   "8",     "7",    "F7",  "R",   "F",    "C",     "STO",
",",   ")",     "(",    "F2",  "T",   "G",    "V",     "Space",
"TAN", "COS",   "SIN",  "F6",  "Y",   "H",    "B",     "/",
"P",   "ENTER2","LN",   "F1",  "U",   "J",    "N",     "^",
"*",   "APPS",  "CLEAR","F5",  "I",   "K",    "M",     "=",
"",    "ESC",   "MODE", "+",   "O",   "L",    "Theta", "BckSpc",
"(-)", ".",     "0",    "F4",  "Q",   "A",    "ENTER1","-"
};
#endif


//========================================================
//    Function: KeyMaskToStr
//
// Description: Translate key identifiers to the key name.
//
//  Parameters: unsigned short - the row number
//              unsigned short - the column number
//
//     Returns: const char* - pointer to the key name
//========================================================
const char* KeyMaskToStr(unsigned short row,
                         unsigned short column)
{
  return key_names[row * 8 + 7-column];
}


//========================================================
//    Function: GetKey
//
// Description: Get the next key pressed.  It will be
//              saved in the configuration file.
//
//  Parameters: unsigned short - store the row number
//              unsigned short - store the column number
//
//     Returns: None
//========================================================
void GetKey(unsigned short *row,
            unsigned short *column)
{
  BOOL pressed = FALSE;
  short loopx = 0;
  short loopy = 0;

  OSSetSR (0x0700);

  // lift off the enter key, lead finger
#ifdef _89
  while(_rowread(~(1<<1)) & (1<<0));
#else
  while(_rowread(~(1<<9)) & (1<<1));
#endif


  // get the key
  while (pressed == FALSE)
  {
  	for (loopx = 0; loopx < Q89_92(7,10); loopx++)
	  {
		  for (loopy = 0; loopy < 8; loopy++)
  		{
	  		if (_rowread(~(1<<loopx)) & (1<<loopy))
		  	{
			  	*row = loopx;
				  *column = loopy;
  				pressed = TRUE;
	  		}
		  }
    }
  }

  OSSetSR (0x0000);
}
