// C Source File
// Created 3/3/2002; 9:50:32 PM

#include "keys.h"
#include "delays.h"
#include "fileio.h"
#include <all.h>


// for contrast changing delay
int contrast_changed = 0;
BOOL special_key = FALSE;

char *key_list[10] = {
	              "move left",
                "move right",
                "move down",
                "rotate clockwise",
                "rotate counter",
                "instant drop",
                "pause/quit",
                "contrast up",
                "contrast down",
                "teacher key"
                };

unsigned short row_list[10] = {0,
                               0,
                               0,
                               0,
                               0,
                               0,
                               Q89_92(6,8),
                               Q89_92(1,8),
                               Q89_92(1,9),
                               Q89_92(3,9)};
                               
unsigned short col_list[10] = {Q89_92(1,4),
                               Q89_92(3,6),
                               Q89_92(2,7),
                               Q89_92(4,0),
                               Q89_92(6,1),
                               Q89_92(5,2),
                               Q89_92(0,6),
                               Q89_92(1,4),
                               Q89_92(2,0),
                               Q89_92(0,6)};


//////////////////////////////////////////////////////////////
//
//  InitKeys
//
void InitKeys()
{
  char *input_key_list[KEY_COUNT];
  unsigned short input_row_list[KEY_COUNT];
  unsigned short input_col_list[KEY_COUNT];
  int count = -1;
  
  if ((count = OpenFile("quexkey", key_list, row_list, col_list)) == -1)
  {
  	// no conf read
  }
  else
  {
    int input_loop = 0;
    int loop = 0;
    for (input_loop = 0; input_loop < count; input_loop++)
    {
    	for (loop = 0; loop < KEY_COUNT; loop++)
    	{
  	  	if (strncmp(input_key_list[input_loop], key_list[loop], strlen(key_list[loop])) == 0)
  		  {
    			// we have a match
    			row_list[loop] = input_row_list[input_loop];
  	  		col_list[loop] = input_col_list[input_loop];
  	    }
  	  }
  	}
  }
}


//////////////////////////////////////////////////////////////
//
//  SaveKeys
//
void SaveKeys()
{
	WriteFile("quexkey", KEY_COUNT, key_list, row_list, col_list);
}


//////////////////////////////////////////////////////////////
//
//  ClearAllKeys
//
void ClearAllKeys()
{
  while (_rowread(0) & 0xff)
  {
  	delay(10);
  }
}


//////////////////////////////////////////////////////////////
//
//  CheckSpecialKeys: contrast and teacher
//
BOOL CheckSpecialKeys()
{
  BOOL teacher_key_pressed = FALSE;

  // have they already pressed the teacher key?
  if (special_key == TRUE)
  {
  	return TRUE;
  }

  // check for contrast up and down
  if(ContrastUpPressed() && contrast_changed == 0)
  {
    // prevent unwanted keys
    delay(UNWANTED_KEY_PRESS_DELAY);

    if(ContrastUpPressed() && (contrast_changed == 0))
    {
      OSContrastUp();
      contrast_changed = 15;
    }
  }
  else if(ContrastDownPressed() && (contrast_changed == 0))
  {
    // prevent unwanted keys
    delay(UNWANTED_KEY_PRESS_DELAY);

    if(ContrastDownPressed() && (contrast_changed == 0))
    {
      OSContrastDn();
      contrast_changed = 15;
    }
  }
  else if(contrast_changed > 0)
  {
    contrast_changed--;
  }

  // return the value of the teacher being pressed
  if(TeacherKeyPressed())
  {
    // prevent unwanted keys
    delay(UNWANTED_KEY_PRESS_DELAY);

    // if the . was pressed, special key = 1
    teacher_key_pressed = TeacherKeyPressed();
    special_key = teacher_key_pressed;
  }

  return teacher_key_pressed;
}

// menu navigation
BOOL UpKeyPressed()
{
  return (_rowread(0xfffe) & Q89_92(1,32));
}

BOOL DownKeyPressed()
{
  return (_rowread(0xfffe) & Q89_92(4,128));
}

BOOL LeftKeyPressed()
{
  return (_rowread(0xfffe) & Q89_92(2,16));
}

BOOL RightKeyPressed()
{
  return (_rowread(0xfffe) & Q89_92(8,64));
}

BOOL EnterKeyPressed()
{
  #ifdef _89
  return (_rowread(0xfffd) & 1);
  #else
  return ((_rowread(0xfdff) & 2) || (_rowread(0xffbf) & 64));
  #endif
}

BOOL EscapeKeyPressed()
{
  return (_rowread(Q89_92(0xffbf,0xfeff)) & Q89_92(1,64));
}


// in game
BOOL MoveLeftPressed()
{
	return (_rowread(~(1<<row_list[0])) & (1<<col_list[0]));
}

BOOL MoveRightPressed()
{
	return (_rowread(~(1<<row_list[1])) & (1<<col_list[1]));
}

BOOL MoveDownPressed()
{
	return (_rowread(~(1<<row_list[2])) & (1<<col_list[2]));
}

BOOL RotateClockwisePressed()
{
	return (_rowread(~(1<<row_list[3])) & (1<<col_list[3]));
}

BOOL RotateCounterPressed()
{
	return (_rowread(~(1<<row_list[4])) & (1<<col_list[4]));
}

BOOL DropKeyPressed()
{
	return (_rowread(~(1<<row_list[5])) & (1<<col_list[5]));
}

BOOL PauseGamePressed()
{
	return (_rowread(~(1<<row_list[6])) & (1<<col_list[6]));
}

BOOL ContrastUpPressed()
{
	return (_rowread(~(1<<row_list[7])) & (1<<col_list[7]));
}

BOOL ContrastDownPressed()
{
	return (_rowread(~(1<<row_list[8])) & (1<<col_list[8]));
}

BOOL TeacherKeyPressed()
{
	return (_rowread(~(1<<row_list[9])) & (1<<col_list[9]));
}

