//===============================================================
//          Queue
// By:      Kurt Hoffmann (kurt@hoffmann.net
// Version: 0.9
// Date:    03-??-02
//===============================================================


#define SAVE_SCREEN

#include <all.h>
#include "queue.h"
#include "keys.h"
#include "delays.h"
#include "drawstuff.h"
#include "network.h"


// the calculator definition is now in keys.h

#if defined(_89)
int _ti89;
#elif defined(_92plus)
int _ti92plus;
#else
#error Target product not defined
#endif

// other definitions
#define LINE_CLEARING_DELAY 12
#define VERSION_NUMBER 9


// global variables
int *board;               // the board (doubled from earlier versions for multiplayer)
int col1 = Q89_92(-9,10); // used in drawing blocks
int wins = 0;             // multiplayer wins
int losses = 0;           // multiplayer losses
unsigned short my_machine = 0;
int turn_off = TURN_OFF_DELAY;

// more global variable just for restore
int x = 0;                // the x position of the block
int y = 0;                // the y position of the block
int rotation = 0;         // the rotation of the block
int icur = 0;             // index of current piece
int ipreview[4];          // index of preview pieces
int ispeed = 0;           // ispeed of pieces (usually based on lines)
int restore = 0;          // restore after special key was pressed
BOOL in_game = FALSE;     // were we in a game when we pressed .?

// numbers to draw on the screen
int ilines = 0;           // lines cleared
int line_totals[10];      // number of times x lines have been cleared
unsigned long iscore = 0; // score (could be > 65536)

// delays are now variable based on the lines
int move_rl_delay = 10; 
int rotate_delay = 20;

// high scores
HSC_ITEM tetris[3]={{"--------", 0, 0},{"--------", 0, 0},{"--------", 0, 0}};
HSC_ITEM quadra[3]={{"--------", 0, 0},{"--------", 0, 0},{"--------", 0, 0}};
HSC_ITEM *current = quadra;

// the pieces
piece pieces0[7] = {{0,4,{0,0,0,0,7,193,13,0,0,14,0,0,0,0,0,0}}, {1,4,{0,0,0,0,131,5,13,0,14,0,0,0,0,0,0,0}}, {2,4,{0,0,0,0,7,5,73,0,0,0,14,0,0,0,0,0}}, {3,2,{0,11,0,0,0,10,0,0,0,10,0,0,0,14,0,0}}, {4,2,{0,0,0,0,7,73,0,0,0,22,13,0,0,0,0,0}}, {5,2,{0,0,0,0,0,131,13,0,7,44,0,0,0,0,0,0}}, {6,1,{0,0,0,0,0,3,9,0,0,6,12,0,0,0,0,0}}};
piece pieces1[6] = {{0,4,{0,11,0,0,7,104,0,0,0,14,0,0,0,0,0,0}}, {1,4,{7,73,0,0,0,10,0,0,0,14,0,0,0,0,0,0}}, {2,4,{0,11,0,0,0,10,0,0,7,44,0,0,0,0,0,0}}, {3,2,{0,0,0,0,7,5,5,13,0,0,0,0,0,0,0,0}}, {4,2,{0,11,0,0,131,44,0,0,14,0,0,0,0,0,0,0}}, {5,2,{11,0,0,0,22,73,0,0,0,14,0,0,0,0,0,0}}};
piece pieces2[3] = {{0,4,{0,11,0,0,7,52,13,0,0,0,0,0,0,0,0,0}}, {1,4,{0,0,11,0,7,5,44,0,0,0,0,0,0,0,0,0}}, {2,4,{11,0,0,0,22,5,13,0,0,0,0,0,0,0,0,0}}};
piece pieces3[3] = {{0,4,{0,11,0,0,0,146,13,0,0,14,0,0,0,0,0,0}}, {1,4,{0,11,0,0,0,10,0,0,0,22,13,0,0,0,0,0}}, {2,4,{0,131,13,0,0,10,0,0,0,14,0,0,0,0,0,0}}};

// for intro
piece *q[22] = {&pieces1[2],&pieces3[0],&pieces1[4],&pieces0[4],&pieces0[6],&pieces1[2],&pieces1[4],&pieces0[6],&pieces1[3],&pieces2[1],&pieces1[5],&pieces3[0],&pieces2[2],&pieces0[4],&pieces1[2],&pieces3[2],&pieces1[3],&pieces0[4],&pieces0[5],&pieces0[6],&pieces0[4],&pieces0[5]};
int title_piece = 0;
int piece_offset = 0;
int piece_color = 1;
int qx[22] = {1,0,2,4,5,6,5,4,4,2,8,7,9,12,12,11,16,17,14,13,14,17};
int qy[22] = {34,32,30,29,31,34,34,36,38,36,33,35,37,36,34,33,33,31,31,33,35,35};



//////////////////////////////////////////////////////////////
//
//  draw_score: I win
//
void draw_score(int style,
                int new_lines)
{
  int i, plane;
  char c[8];

  for (plane = 0; plane < 2; plane++)
  {
    SetPlane(plane);

    // print lines and score
    #if defined(_89)
    FontSetSys(F_6x8);
    #elif defined(_92plus)
    FontSetSys(F_8x10);
    #endif

    sprintf(c, "%d", ilines);
    DrawStr(Q89_92(130,183), Q89_92(69,90), c, A_REPLACE);
    sprintf(c, "%lu", iscore);
    DrawStr(Q89_92(116,169), Q89_92(90,114), c, A_REPLACE);

    // print the line totals
    #if defined(_89)
    FontSetSys(F_4x6);
    #elif defined(_92plus)
    FontSetSys(F_6x8);
    #endif

    for (i = 0; i < 4 + 6*style; i++)
    {
      sprintf(c, "%d", line_totals[i]);
      DrawStr(Q89_92(14,20), Q89_92(9,12) + i * Q89_92(6,8), c, A_REPLACE);
    }

    draw_line_total_numbers(style, new_lines);
  }
}


//////////////////////////////////////////////////////////////////
//
// exit_window: small function mostly for multiplayer messages
//
void exit_window(char* message)
{
  int draw_score = 0;

  // if multiplayer and just finished a game, draw the score
  if (getMachine() != NEITHER && in_game == TRUE)
  {
  	draw_score = 1;
  }

  // erase dark layer (plane 1)
  memset(GetPlane(1),0,LCD_SIZE);

  // window
  draw_window(Q89_92(30,70),Q89_92(10,21),Q89_92(130,170),Q89_92(40,53) + 15*draw_score,message, 1);

  // draw words
  SetPlane(1);
  FontSetSys(F_6x8);
  DrawStr(Q89_92(67,107),Q89_92(24,37),"Exit",A_NORMAL);

  // draw box
  draw_box(Q89_92(40,80), Q89_92(22,35), Q89_92(120,160), Q89_92(33,46), 1);

  // draw the actual score
  if (draw_score)
  {
    for (draw_score = 0; draw_score < 2; draw_score++)
    {
      SetPlane(draw_score);
      FontSetSys(F_6x8);
      printf_xy(Q89_92(65,105),Q89_92(40,53),"%d - %d", wins, losses);
    }
  }

  // wait until they press escape or enter
  turn_off = TURN_OFF_DELAY;
  while (EnterKeyPressed() == FALSE && EscapeKeyPressed() == FALSE)
  {
    turn_off = check_off(turn_off);
  }
  turn_off = TURN_OFF_DELAY;
}


//////////////////////////////////////////////////////////////
//
//  update_block_histogram: histogram origionally by Alex Astashyn
//  precondition: graymode is assumed to be on.
//
void update_block_histogram(int values[],
                            int n)
{
	int i;
	int max_index = 0;
	int length;

	values[n]++;

	for (i = 1; i < 7; i++)
	{
		if(values[i] > values[max_index])
		{
			max_index = i;
    }
  }

	for (i = 0; i < 7; i++)
	{
		length = (Q89_92(9,16) + 1) * values[i] / values[max_index];
		draw_bar(
			Q89_92(95,121) + i * Q89_92(9,17), 
			Q89_92(41,51),
			Q89_92(9,16),
			Q89_92(3,7),
			length == 0 && values[i] != 0 ? 1 : length);
	}
}


//////////////////////////////////////////////////////////////
//
//  hsc_RW: origionally by Alex Astashyn
//
void hsc_RW(int *pstyle,
            int *pshadow,
            int *pspeed,
            int *pjunk,
            BOOL write)
{
	int i;
	FILE *phile;

	if (write == TRUE)
	{
		// unarchive (if archived)
		EM_moveSymFromExtMem($(quexhs), HS_NULL); 

		phile = fopen("quexhs", "wb");

    // version protection
		i = VERSION_NUMBER;
		fwrite(&i, sizeof(int), 1, phile);
    fwrite(tetris, 3*sizeof(HSC_ITEM), 1, phile);
    fwrite(quadra, 3*sizeof(HSC_ITEM), 1, phile);
		fwrite(pstyle, sizeof(int), 1, phile);
		fwrite(pshadow, sizeof(int), 1, phile);
		fwrite(pspeed, sizeof(int), 1, phile);
		fwrite(pjunk, sizeof(int), 1, phile);

    // restore?
    i = (CheckSpecialKeys() == TRUE) && (in_game == TRUE) && (getMachine() == NEITHER);
    fwrite(&i, sizeof(int), 1, phile);

    // variables for restore
    fwrite(board, 510*sizeof(int), 1, phile);
    fwrite(&x, sizeof(int), 1, phile);
    fwrite(&y, sizeof(int), 1, phile);
    fwrite(&rotation, sizeof(int), 1, phile);
    fwrite(&icur, sizeof(int), 1, phile);
    fwrite(ipreview, 3*sizeof(int), 1, phile);
    fwrite(line_totals, 10*sizeof(int), 1, phile);
    fwrite(&ilines, sizeof(int), 1, phile);

    // correct score
    iscore--;
    fwrite(&iscore, sizeof(unsigned long), 1, phile);

		fputc(0, phile);
		fputs("INI", phile);
		fputc(0, phile);
		fputc(OTH_TAG, phile);
		fclose(phile);
	}
	else
	{
		phile = fopen("quexhs", "rb");

		if (phile != NULL)
		{
      // version protection
      fread(&i, sizeof(int), 1, phile);

      if (i == VERSION_NUMBER)
      {
        fread(tetris, 3*sizeof(HSC_ITEM), 1, phile);
        fread(quadra, 3*sizeof(HSC_ITEM), 1, phile);
				fread(pstyle, sizeof(int), 1, phile);
				fread(pshadow, sizeof(int), 1, phile);
				fread(pspeed, sizeof(int), 1, phile);
				fread(pjunk, sizeof(int), 1, phile);

        // did they do a quick exit?
        fread(&restore, sizeof(int), 1, phile);

        if (restore == TRUE)
        {
          // read in board
          fread(board, 510*sizeof(int), 1, phile);

          // more restore variables
          fread(&x, sizeof(int), 1, phile);
          fread(&y, sizeof(int), 1, phile);
          fread(&rotation, sizeof(int), 1, phile);
          fread(&icur, sizeof(int), 1, phile);
          fread(ipreview, 3*sizeof(int), 1, phile);
          fread(line_totals, 10*sizeof(int), 1, phile);
          fread(&ilines, sizeof(int), 1, phile);
          fread(&iscore, sizeof(unsigned long), 1, phile);

        }
			}
		}
		fclose(phile);
	}
}

//////////////////////////////////////////////////////////////
//
//  input_str: by Zeljko Juric
//
void input_str(char *str,
               int x,
               int y,
               int maxlen)
{
  int i=0,key=0;
  while (key != 13)
  {
      str[i]='_'; str[i+1]=' '; str[i+2]=0;
      DrawStr(x,y,str, A_REPLACE);
      key=ngetchx();
      if(key>=' '&&key<='~'&&i<maxlen) str[i++]=key;
      if(key==257&&i) i--;
  }
  str[i]=0;
}

//////////////////////////////////////////////////////////////
//
//  update_hsc: originally by Zeljko Juric
//
void update_hsc()
{
  int i;

  // clear the screen
  ClrScr();

  for (i = 2; (current[i-1].score < iscore) && i; i--)
  {
    memcpy(&current[i],&current[i-1],sizeof(HSC_ITEM));
  }

  current[i].lines = ilines;
  current[i].score = iscore;

  FontSetSys(F_8x10);
  DrawStr(Q89_92(16,56),Q89_92(20,34),"Congratulations!", A_NORMAL);  
  FontSetSys(F_6x8);
  DrawStr(Q89_92(25,65),Q89_92(35,49),"You placed", A_NORMAL);

  // display first, second, or third depending on their place
  if (i == 2)
  {
  	DrawStr(Q89_92(90,130),Q89_92(35,49),"third!", A_NORMAL);
  }
  else if (i == 1)
  {
  	DrawStr(Q89_92(90,130),Q89_92(35,49),"second!", A_NORMAL);
  }
  else if (i == 0)
  {
  	DrawStr(Q89_92(90,130),Q89_92(35,49),"first!", A_NORMAL);
  }

  DrawStr(Q89_92(30,70),Q89_92(60,74),"Enter your name", A_NORMAL);
  DrawStr(Q89_92(20,60),Q89_92(70,84),"(max. 8 characters):", A_NORMAL);

  input_str(current[i].name,Q89_92(50,90),Q89_92(84,98),8);
}


//////////////////////////////////////////////////////////////
//
//  draw_quit: do you want to quit?
//
void draw_quit(int quit)
{
  int i;

  FontSetSys(F_6x8);

  // draw words
  SetPlane(1);

  for (i = Q89_92(22,35); i < Q89_92(48,61); i++)
  {
  	DrawLine(Q89_92(40,80),i,Q89_92(120,160),i,A_REVERSE);
  }

  DrawStr(Q89_92(56,96),Q89_92(24,37),"Continue",A_NORMAL);
  DrawStr(Q89_92(67,107),Q89_92(37,50),"Quit",A_NORMAL);

  // erase old highlight
  SetPlane(0);
  for (i = Q89_92(22,35); i < Q89_92(48,61); i++)
  {
  	DrawLine(Q89_92(40,80),i,Q89_92(120,160),i,A_REVERSE);
  }

  // draw box
  draw_box(Q89_92(40,80), Q89_92(22,35)+13*quit, Q89_92(120,160), Q89_92(33,46)+13*quit, 1);
}


//////////////////////////////////////////////////////////////
//
//  draw_options
//
void draw_options(int style,
                  int shadow,
                  int speed,
                  int junk,
                  int row)
{
  int i, plane;
  char* temp = "0";

  // clear out old stuff
  FontSetSys(F_4x6);
  for (plane = 0; plane < 2; plane++)
  {
    SetPlane(plane);

    for (i = 0; i < 32; i++)
    {
    	DrawLine(Q89_92(10,50),Q89_92(62,82)+i,Q89_92(150,190),Q89_92(62,82)+i,A_REVERSE);
    }

    DrawStr(Q89_92(47,87),Q89_92(64,84),"Mode:", A_NORMAL);
    DrawStr(Q89_92(39,79),Q89_92(72,92),"Shadow:", A_NORMAL);
    DrawStr(Q89_92(15,55),Q89_92(80,100),"Starting Speed:", A_NORMAL);
    if (!style)
    {
    	DrawStr(Q89_92(19,59),Q89_92(88,108),"Starting Junk:", A_NORMAL);
    }
  }

  // draw all the options on plane 1
  DrawStr(Q89_92(72,112),Q89_92(64,84),"Tetris",A_NORMAL);
  DrawStr(Q89_92(102,142),Q89_92(64,84),"Quadra",A_NORMAL);
  DrawStr(Q89_92(80,120),Q89_92(72,92),"No",A_NORMAL);
  DrawStr(Q89_92(110,150),Q89_92(72,92),"Yes",A_NORMAL);

  // speed and junk
  for (i = 0; i < 10; i++)
  {
    temp[0] = 48 + i;
    DrawStr(Q89_92(71,111)+8*i,Q89_92(80,100),temp,A_NORMAL);
    if (!style)
    {
    	DrawStr(Q89_92(71,111)+8*i,Q89_92(88,108),temp,A_NORMAL);
    }
  }

  // draw boxes
  draw_box(Q89_92(70,110) + 30*style, Q89_92(63,83), Q89_92(100,140) + 30*style, Q89_92(69,89), (row==0));
  draw_box(Q89_92(70,110) + 30*shadow, Q89_92(71,91), Q89_92(100,140) + 30*shadow, Q89_92(77,97), (row==1));
  draw_box(Q89_92(70,110) + 8*speed, Q89_92(79,99), Q89_92(76,116) + 8*speed, Q89_92(85,105), (row==2));
  if (!style)
  {
  	draw_box(Q89_92(70,110) + 8*junk, Q89_92(87,107), Q89_92(76,116) + 8*junk, Q89_92(93,113), (row==3));
  }
}


//////////////////////////////////////////////////////////////
//
//  draw_high_scores
//
void draw_high_scores()
{
  int plane, i;
  char c[10];

  // set the font
  FontSetSys(F_4x6);

  // draw the high scores
  for (plane = 0; plane < 2; plane++)
  {
    SetPlane(plane);

    // erase the area
    for (i = Q89_92(78,98); i < Q89_92(100,125); i++)
    {
    	DrawLine(Q89_92(0,40),i,Q89_92(159,200),i,A_REVERSE);
    }

    // draw high scores
    for (i = 0; i < 3; i++)
    {
      sprintf(c, "%d)%s", i+1, current[i].name);
      DrawStr(Q89_92(15,55), Q89_92(79,100) + 8*i, c, A_REPLACE);
      sprintf(c, "%ld", current[i].lines);
      DrawStr(Q89_92(90,130), Q89_92(79,100) + 8*i, c, A_REPLACE);
      sprintf(c, "%ld", current[i].score);
      DrawStr(Q89_92(120,160), Q89_92(79,100) + 8*i, c, A_REPLACE);
    }
  }
}


//////////////////////////////////////////////////////////////
//
//  test_position: can I move there?
//
int test_position(int x,
                  int y,
                  const piece* const test)
{
  int i,j;
  for (i = 0; i < 4; i++)
  {
    for (j = 0; j < 4; j++)
    {
      // make sure it is in bounds
      if ((test->blocks[4*j+i] != 0) && ((i+x > 9) || (i+x < 0) || (j+y > 49)))
      {
        return 0;
      }

      // make sure there is not a piece there
      if ((test->blocks[4*j+i] != 0) && (board[10*(j+y) + (i+x)] != 0))
      {
        return 0;
      }
    }
  }
  return 1;
}

//////////////////////////////////////////////////////////////
//
//  draw_block: some day this will be with bitmaps
//
void draw_block(int x,
                int y,
                int block_num)
{
   // block_num == 0 means erase the block
   // otherwise draw based on block_num

  int i, j;

  // correct y for bigger board
  y -= 25;

  // only draw if the block is on the screen
  if (y < Q89_92(5,4))
  {
  	return;
  }

  // first erase if requested
  if (block_num == 0)
  {
    for (i = 0; i < 2; i++)
    {
      SetPlane(i);
      for (j = 0; j < Q89_92(5,6); j++)
      {
      	DrawLine(col1+Q89_92(5,6)*x+j+Q89_92(5,6), Q89_92(5,6)*y-Q89_92(25,25), col1+Q89_92(5,6)*x+j+Q89_92(5,6), Q89_92(5,6)*y-Q89_92(21,20), A_REVERSE);
      }
    }
    return;
  }

  // erase shadow
  SetPlane(0);
  for (i=0; i < Q89_92(5,6); i++)
  {
  	DrawLine(col1+Q89_92(5,6)*x+i+Q89_92(5,6), Q89_92(5,6)*y-Q89_92(25,25),  col1+Q89_92(5,6)*x+i+Q89_92(5,6), Q89_92(5,6)*y-Q89_92(21,20), A_REVERSE);
  }

  // now we are on plane 0

  // bottom
  if (block_num & 4)
  {
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(6,7), Q89_92(5,6)*y - Q89_92(21,20), col1+Q89_92(5,6)*x + Q89_92(8,10), Q89_92(5,6)*y - Q89_92(21,20), A_NORMAL);
  }

  // right
  if (block_num & 8)
  {
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(24,24), col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(22,21), A_NORMAL);
  }

  // if a line in bottom right make it black, else leave it gray
  if ((block_num & 4) || (block_num & 8) || (block_num & 128))
  {
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(21,20), col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(21,20), A_NORMAL);
  }

  // upper right light, dark, or gray
  if ((block_num & 1) && !(block_num & 8))
  {
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);
  }
  else if ((block_num & 8) && !(block_num & 1))
  {
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);
    SetPlane(1);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);
  }
  else
  {
    SetPlane(1);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(9,11), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);
  }

  // plane unknown

  // middle
  SetPlane(1);
  for (i=1; i < Q89_92(5,6); i++)
  {
  	DrawLine(col1+Q89_92(5,6)*x+i+Q89_92(5,6), Q89_92(5,6)*y-Q89_92(24,24),  col1+Q89_92(5,6)*x+i+Q89_92(5,6), Q89_92(5,6)*y-Q89_92(21,20), A_NORMAL);
  }

  // top
  SetPlane((block_num&1) == 0);
  DrawLine(col1+Q89_92(5,6)*x + Q89_92(6,7), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(8,10), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);
  
  // left
  SetPlane((block_num&2) == 0);
  DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(24,24), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(22,21), A_NORMAL);

  // if a line in upper left make it light, else make it gray
  SetPlane(((block_num&1) == 0) && ((block_num&2) == 0));
  DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);

  // lower left light, dark, or gray
  if ((block_num & 2) && !(block_num & 4))
  {
    SetPlane(0);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), A_NORMAL);
  }
  else if ((block_num & 4) && !(block_num & 2))
  {
    SetPlane(0);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), A_NORMAL);
    SetPlane(1);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), A_NORMAL);
  }
  else
  {
    SetPlane(1);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(21,20), A_NORMAL);
  }

  // upper left
  if (block_num & 32)
  {
    SetPlane(1);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(25,25), A_REVERSE);
    SetPlane(0);
    DrawLine(col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(25,25), col1+Q89_92(5,6)*x + Q89_92(5,6), Q89_92(5,6)*y - Q89_92(25,25), A_NORMAL);  
  }
}

//////////////////////////////////////////////////////////////
//
//  draw_piece
//
int draw_piece(int x,
               int y,
               const piece* const cur,
               int style)
{
  // style = 0 means erase
  // style = 1 means normal
  // style = 2 means shadow
  // style = 3 means put piece

  int i, j, k;

  for (i = 0; i < 4; i++)
  {
    for (j = 0; j < 4; j++)
    {
      if (cur->blocks[4*j+i])
      {
        // if there is already something there, we have overlap = we are dead
        if ((x < 11) && board[10*(j+y) + (i+x)])
        {
        	return 0;
        }

        if (style == 0)
        {
          draw_block(i+x,j+y, 0);
        }
        else if (style == 1)
        {
          draw_block(i+x,j+y,cur->blocks[4*j+i]);
        }
        else if (style == 2)
        {
          for (k = 0; k < Q89_92(5,6); k++)
          {
            DrawLine(Q89_92(33,46)+Q89_92(5,6)*(x+i)+k, Q89_92(5,6)*(y+j)-Q89_92(150,175),  Q89_92(33,46)+Q89_92(5,6)*(x+i)+k, Q89_92(5,6)*(y+j)-Q89_92(146,170), A_NORMAL);
          }
        }
        else
        {
          // put the block there
          board[10*(j+y) + (i+x)] = cur->blocks[4*j+i];
        }
      }
    }
  }

  return 1;
}


//////////////////////////////////////////////////////////////////
//
// do_gravity(): the heart and soul of quadra
//
void do_gravity()
{
  int i, j, k, moved = 0, can_drop;
  int iblocks, iblocks_x[40], iblocks_y[40];
  int verticle, can_go_verticle, right, can_go_right, left, can_go_left;
  do
  {
    // line clearing delay note: (ispeed = 3*(lines/10))
    delay(LINE_CLEARING_DELAY - ispeed / 5);

    moved = 0;
    for (j = 48; j > 2; j--)
    {
      for (i = 0; i < 10; i++)
      {
        // iblocks is the number of blocks in the current piece
        iblocks = 0;
        can_drop = 1;

        // we can not move down at all unless there is a space under the current block
        // AND there is nothing attached to the left (it would be a part of a different piece)
        if ((board[10*j+i] != 0) && (board[10*(j+1) + i] == 0) && (board[10*j+i]&2))
        {
          // make sure we are on the bottom row of this block... we only can
          // test from the bottom row
          can_go_right = 1;
          for (right = 0; can_go_right == 1; right++)
          {
            // test for floor
            if (!(board[10*j+i+right]&4))
            {
              // no floor! we got problems
              can_drop = 0;
              can_go_right = 0;
            }
          
            // test for right wall
            if (board[10*j+i+right]&8)
            {
              // we have reached the wall
              can_go_right = 0;
            }
          }

          // ok, we might be able to move the current block down
          iblocks_x[iblocks] = i;
          iblocks_y[iblocks] = j;
          iblocks++;
          
          // master piece finding loop
          can_go_verticle = 1;
          for (verticle = 0; can_go_verticle == 1; verticle++)
          {
            // loop to the right
            can_go_right = 1;
            for (right = 0; can_go_right == 1; right++)
            {
              // check right bar
              if ((i+right+1 > 9) || (board[10*(j-verticle)+(i+right)]&8))
              {
                can_go_right = 0;
                
                // special case!  we could have pieces above this
                
                if ((right != 0) && !(board[10*(j-verticle)+(i+right)]&1) && (board[10*(j-verticle-1)+(i+right)]&2))
                {
                  // we have one above this right one
                  iblocks_x[iblocks] = i+right;
                  iblocks_y[iblocks] = j-verticle-1;
                  iblocks++;
                  
                  // special case!  we could have two pieces above
                
                  if ((right != 0) && !(board[10*(j-verticle-1)+(i+right)]&1) && (board[10*(j-verticle-2)+(i+right)]&2))
                  {
                    // we have one above this right one
                    iblocks_x[iblocks] = i+right;
                    iblocks_y[iblocks] = j-verticle-2;
                    iblocks++;
                  } 

                  // special case!  we could have a pieces above and to the right

                  if ((right != 0) && !(board[10*(j-verticle-1)+(i+right)]&8) && (board[10*(j-verticle-1)+(i+right)]&2))
                  {
                    if (board[10*(j-verticle)+(i+right+1)] == 0)
                    {
                      // we have one above this right one
                      iblocks_x[iblocks] = i+right+1;
                      iblocks_y[iblocks] = j-verticle-1;
                      iblocks++;
                    }
                    else
                    {
                      can_drop = 0;
                    }
                  }
                }

                // end special cases

              }
              else
              {
                // we can go right
                
                // special case
                if ((right != 0) && !(board[10*(j-verticle)+(i+right)]&1) && (board[10*(j-verticle-1)+(i+right)]&2))
                {
                  // we have one above this right one
                  iblocks_x[iblocks] = i+right;
                  iblocks_y[iblocks] = j-verticle-1;
                  iblocks++;
                }
                                
                // make sure it can drop
                if ((board[10*(j-verticle+1)+(i+right+1)] == 0) || !(board[10*(j-verticle)+(i+right+1)]&4))
                {
                  iblocks_x[iblocks] = i+right+1;
                  iblocks_y[iblocks] = j-verticle;
                  iblocks++;
                }
                else
                {
                  can_drop = 0;
                  can_go_right = 0;
                }
              
              }
            
            }

            // loop to the left
            can_go_left = 1;
            for (left = 0; can_go_left == 1; left++)
            {
              // check left bar
              if ((i-left-1 < 0) || (board[10*(j-verticle)+(i-left)]&2))
              {
                can_go_left = 0;
                
                // special case!  we could have pieces above this
                
                if ((left != 0) && !(board[10*(j-verticle)+(i-left)]&1) && (board[10*(j-verticle-1)+(i-left)]&2))
                {
                  // we have one above this right one
                  iblocks_x[iblocks] = i-left;
                  iblocks_y[iblocks] = j-verticle-1;
                  iblocks++;
                }
              }
              else
              {
                // we can go left
                
                // make sure it can drop
                if ((board[10*(j-verticle+1)+(i-left-1)] == 0) || !(board[10*(j-verticle)+(i-left-1)]&4))
                {
                  iblocks_x[iblocks] = i-left-1;
                  iblocks_y[iblocks] = j-verticle;
                  iblocks++;
                }
                else
                {
                  can_drop = 0;
                  can_go_left = 0;
                }
              
              }
            
            } 
            
            // check for top bar
            if (board[10*(j-verticle)+i] & 1)
            {
              can_go_verticle = 0;
            }
            else
            {
              // we can go up, add the upper piece
              iblocks_x[iblocks] = i;
              iblocks_y[iblocks] = j-verticle-1;
              iblocks++;            
            }

          }
        
        }
        else
        {
          // we have no chance of moving the current piece down
          iblocks = 0;
        }

        // move blocks if you can
        if ((iblocks != 0) && (can_drop == 1))
        {
          // we are gonna move something
          moved = 1;

          // do the actual moving
          for (k = 0; k < iblocks; k++)
          {
            // draw the blocks of the new position
            draw_block(iblocks_x[k], iblocks_y[k] + 1, board[10*(iblocks_y[k]) + iblocks_x[k]]);
            board[10*(iblocks_y[k]+1) + iblocks_x[k]] = board[10*(iblocks_y[k]) + iblocks_x[k]];
          
            // erase blocks of the old position
            draw_block(iblocks_x[k], iblocks_y[k], 0);
            board[10*iblocks_y[k] + iblocks_x[k]] = 0;
          }        
        }
      
      // inside for loops
      }
    }
  } while (moved != 0);
}


//////////////////////////////////////////////////////////////////
//
// test_for_lines(): the heart and soul of tetris (and part of quadra)
//
int test_for_lines()
{
  // A_REVERSE = 0
  // A_NORMAL = 1
  // A_REPLACE = 4

  int i, j, holes[50], lines = 0, lowest_line = 0, loop;
  SCR_RECT rect;

  // constants
  rect.xy.x0 = Q89_92(33,46);
  rect.xy.x1 = Q89_92(82,105);

  // determine complete lines
  for (j = 0; j < 50; j++)
  {
    holes[j] = 0;

    // determine if there are any holes on this line
    for (i = 0; i < 10; i++)
    {
      holes[j] += (board[10*j+i] == 0);
    }

    // if there were no holes, 
    lines += (holes[j] == 0);
  }

  // flash them
  if (lines != 0)
  {
    for (loop = 0; loop < 4; loop++)
    {
      for (j = Q89_92(30,29); j < 50; j++)
      {
        if (!holes[j])
        {
          FlashLine(j, A_NORMAL-(loop%2));
        }
      }

      // line clearing delay note: (ispeed = 3*(lines/10))
      delay(LINE_CLEARING_DELAY - ispeed / 5);
    }
  }

  // fix variable board
  for (j = 0; j < 50; j++)
  {
    if (!holes[j])
    {
      // fix blocks above and below
      for (i = 0; i < 10; i++)
      {
        board[10*(j+1)+i] |= (board[10*(j+1)+i] != 0);
        board[10*(j-1)+i] |= 4 * (board[10*(j-1)+i] != 0);
      }

      // move all board pieces above it down one
      for (i = 10*j+9; i > 10; i--)
      {
        board[i] = board[i-10];
      }
    }
  }

  // fix and redraw the board
  for (j = 49; j > 0; j--)
  {
    if (!holes[j])
    {
      lowest_line = j;

      // fix lines in board array
      while (!holes[j] && j > 0) j--;

      // number of lines to clear = lowest_line - j
      // calculate lines to move
      rect.xy.y0 = Q89_92(5,6) * (lowest_line-Q89_92(30,29)) - Q89_92(0,1);
      rect.xy.y1 = Q89_92(5,6) * (lowest_line-Q89_92(29,28)) + (lowest_line != 49) - Q89_92(1,2);

      // clear two lines with mutant pieces
      EraseRect(rect);

      // redraw mutant pieces
      for (i = 0; i < 10; i++)
      {
        draw_block(i,lowest_line,board[10*lowest_line+i]);
        if(lowest_line != 49)
        {
          draw_block(i,lowest_line+1,board[10*(lowest_line+1)+i]);
        }
      }

      // prepare to move stuff down
      rect.xy.y0 = 0;
      rect.xy.y1 = Q89_92(5,6) * (lowest_line - Q89_92(30,29)) - 1;

      // fix problem with lines on the top of or above the screen
      if (lowest_line < Q89_92(31,30))
      {
      	rect.xy.y1 = 0;
      }

      // move stuff down
      MoveRect(rect, j - lowest_line);

      // holes matrix needs to be fixed also
      for (i = 49; i > lowest_line - j; i--)
      {
      	holes[i] = holes[i-(lowest_line - j)];
      }
      j = lowest_line;
    }
  }

  // new blocks at top
  for (i = 0; i < 10; i++)
  {
    for (loop = Q89_92(30,29); loop < Q89_92(31,30) + lines; loop++)
    {
      draw_block(i,loop,board[10*loop+i]);
    }
  }

  return lines;
}


//////////////////////////////////////////////////////////////
//
//  board_up: when you send lines in multiplayer
//
void board_up(int sent_lines)
{
  int i, j, top_junk[10], middle_junk[10], bottom_junk[10], hole;

  if (sent_lines != 0)
  {
    // create new junk
    top_junk[0] = 3;
    middle_junk[0] = 2;
    bottom_junk[0] = 6;
    top_junk[9] = 9;
    middle_junk[9] = 8;
    bottom_junk[9] = 12;

    for (i = 0; i < 10; i++)
    {
    	top_junk[i] = 1;
    	middle_junk[i] = 64;
    	bottom_junk[i] = 4;
    }

    hole = random(10);

    top_junk[hole] = 0;
    middle_junk[hole] = 0;
    bottom_junk[hole] = 0;

    if (hole == 0)
    {
    	top_junk[1] = 3;
    	middle_junk[1] = 2;
    	bottom_junk[1] = 6;
    }
    else if (hole == 9)
    {
      top_junk[8] = 9;
      middle_junk[8] = 8;
      bottom_junk[8] = 12;
    }
    else
    {
    	top_junk[hole+1] = 3;
    	middle_junk[hole+1] = 2;
    	bottom_junk[hole+1] = 6;
      top_junk[hole-1] = 9;
      middle_junk[hole-1] = 8;
      bottom_junk[hole-1] = 12;
    }

    // move things up
  	for (j = 0; j < 50 - sent_lines; j++)
    {
    	for (i = 0; i < 10; i++)
    	{
    		board[10*j+i] = board[10*(j+sent_lines)+i];
    	}
    }

    // add in the junk
    for (j = 50 - sent_lines; j < 50; j++)
    {
      for (i = 0; i < 10; i++)
      {
        board[10*j+i] = middle_junk[i];
      }
    }

    // pretty tops and bottoms
    for (i = 0; i < 10; i++)
    {
    	board[10*(50-sent_lines)+i] |= top_junk[i];
    	board[490+i] |= bottom_junk[i];
    }

    // redraw board
    for (j = 49; j > 28; j--)
    {
      for (i = 0; i < 10; i++)
      {
        draw_block(i,j, 0);
        if (board[10*j + i] != 0)
        {
          draw_block(i, j, board[10*j+i]);
        }
      }
    }

    // redo holes on side for multiplayer
    SetPlane(1);
    for (i = 0; i < Q89_92(5,6); i++)
    {
      DrawLine(Q89_92(83,106)+i,0,Q89_92(83,106)+i,Q89_92(99,127),A_NORMAL);
    }
  }
}


//////////////////////////////////////////////////////////////////
//
// draw the board
//
void draw_board()
{
  int i, j;
  for (j = 50; j > 28; j--)
  {
    for (i = 0; i < 10; i++)
    {
      if (board[10*j + i] != 0)
      {
        draw_block(i, j, board[10*j+i]);
      }
    }
  }
}


//////////////////////////////////////////////////////////////////
//
// game: the heart and soul of Queue
//
unsigned long game(int style,
                   int shadow,
                   int speed,
                   int junk)
{
  int i, new_x, new_y, new_rotation, first_loop, quit = 0, sent_lines = 0;
  int redraw, alive=1, moving=0, count=0, new_lines=0, clear = 0;
  int can_move_lr = 1, first_move_lr = 2, can_rotate = 1, first_rotate = 2, can_move_down = 1;
  unsigned long result=0;
  int block_stats[7] = {0};
  char buffer[2];
	unsigned int scoretable[16]={0,40,100,300,1200,5000,6000,7000,8000,9000,10000,11000,12000,13000,14000,15000};
  piece *new_piece, *cur, *preview[3];

  // screen rect for clear screen bonus
  SCR_RECT rect;
  rect.xy.x0 = Q89_92(33,46);
  rect.xy.y0 = Q89_92(50,60);
  rect.xy.x1 = Q89_92(82,105);
  rect.xy.y1 = Q89_92(80,100);

  // sync if in multiplayer mode
  if (getMachine() != NEITHER)
  {
    // clear enter
    ClearAllKeys();

    // clear plane 1
    memset(GetPlane(1),0,LCD_SIZE);

    // draw window
    draw_window(Q89_92(2,42),Q89_92(51,69),Q89_92(158,198),Q89_92(77,97),"Connecting...",1);

    // draw words
    SetPlane(1);
    FontSetSys(F_6x8);
    DrawStr(Q89_92(40,80),Q89_92(64,84),"Connecting...",A_NORMAL);

    while (getMachine() != NEITHER && networkReady() == FALSE && EscapeKeyPressed() == FALSE)
    {
    	delay(10);
    }

    // if we got an error or pressed escape
    if (getMachine() == NEITHER || EscapeKeyPressed() == TRUE)
    {
      return result;
    }

    if (getMachine() == SERVER)
    {
      delay(10);

    	// connected as master
    	buffer[0] = (char)style + 10;
      LIO_SendData(buffer, 1);
    	buffer[0] = (char)shadow + 10;
      LIO_SendData(buffer, 1);
    	buffer[0] = (char)speed + 10;
      LIO_SendData(buffer, 1);
    	buffer[0] = (char)junk + 10;
      LIO_SendData(buffer, 1);

      // make both calcs have the same pieces
    	buffer[0] = (char)random(100) + 10;
      LIO_SendData(buffer, 1);
      srand(buffer[0]);
    }
    else
    {
    	// connected as slave
    	LIO_RecvData(buffer, 1, 20);
      style = (int)(buffer[0]) - 10;
    	LIO_RecvData(buffer, 1, 20);
      shadow = (int)(buffer[0]) - 10;
    	LIO_RecvData(buffer, 1, 20);
      speed = (int)(buffer[0]) - 10;
    	LIO_RecvData(buffer, 1, 20);
      junk = (int)(buffer[0]) - 10;
    	LIO_RecvData(buffer, 1, 20);
      srand(buffer[0]);
    }
  }
  else
  {
    // clear wins and losses
  	wins = 0;
    losses = 0;
  }


  // ispeed based on input starting speed (0-9)
  ispeed = 3 * speed;

  // draw the screen
  if ((style != 0) && (style != 1)) style = 1;
  {
  	draw_screen(style);
  }

  // cur (for restore)
  if (rotation == 0)
  {
  	cur = &pieces0[icur];
  }
  else if (rotation == 1)
  {
  	cur = &pieces1[icur];
  }
  else if (rotation == 2)
  {
  	cur = &pieces2[icur];
  }
  else
  {
  	cur = &pieces3[icur];
  }

  // are we restoring a game...
  if (restore)
  {
  	draw_board();

    // fill preview
    for (i = 0; i < 3; i++)
    {
    	preview[i] = &pieces0[ipreview[i]];
    }
  }
  else
  {
    // fill preview
    for (i = 0; i < 3; i++)
    {
      ipreview[i] = random(7);
      preview[i] = &pieces0[ipreview[i]];
    }

    // clear the board
    for (i = 0; i < 510; i++)
      board[i] = 0;

    // clear line totals
    for (i = 0; i < 10; i++)
      line_totals[i] = 0;

    // add junk if necessary
    while ((!style) && (junk-- != 0))
    {
      for (i = 0; i < 10; i++)
      {
        if (random(2))
        {
          board[500-junk*10+i] = 239;
          draw_block(i,25-junk,239);
        }
      }
    }

    // draw the board (thanks TM)
    draw_board();

    // other vars
    ilines = 0;
    iscore = 0;
  }

  // going into the game
  in_game = TRUE;

  while (alive && !CheckSpecialKeys())
  {
    moving = 1;
    redraw = 1;
    first_loop = 1;
    new_lines = 0;

    // get the lines.  If it is quadra mode, do gravity and recheck the lines...
    if (style)
    {
      for (i = test_for_lines(); i != 0; i = test_for_lines())
      {
        do_gravity();
        new_lines += i;
      }
    }
    else
    {
      new_lines =  test_for_lines();
    }

    // send and receive multiplayer information
    if (getMachine() != NEITHER)
    {
      // move us up if necessary
      if(sent_lines != 0)
      {
        board_up(sent_lines);
        sent_lines = 0;

        if(style)
        {
          do_gravity();

          for(i = test_for_lines(); i != 0; i = test_for_lines())
          {
            do_gravity();
            new_lines += i;
          }
        }

      }

      // send new lines
      if (new_lines != 0)
      {
        buffer[0] = (char)(new_lines + 49);
        OSWriteLinkBlock(buffer,1);
      }

      // send block height information
      for (i = 0; i < 500 && (board[i]==0); i++);
      i = 50 - i/10;
      if (i > 20)
      {
      	i = 20;
      }
      buffer[0] = (char)(i + 100);
      OSWriteLinkBlock(buffer,1);
    }

    // incriment the line counter array
    if (new_lines < 10)
    {
      line_totals[new_lines-1]++;
    }
    else
    {
      line_totals[9]++;
    }

    // check for clear screen bonus
    for (i = 490; (i < 501) && (board[i] == 0); i++);
    if ((i == 501) && (iscore > 1))
    {
      FontSetSys(F_4x6);
      DrawStr(Q89_92(36,51), Q89_92(50,65), "clear screen", A_NORMAL);
      DrawStr(Q89_92(38,53), Q89_92(57,72), "+500 points", A_NORMAL);
      clear = 1;
      iscore += 500;
    }

    // update the number of lines
    ilines += new_lines;

    // SPEED based on number of lines/10
    //                ( 0 - 12) => ( 0 - 36)
    //                (13 - 16) => (38 - 44)
    //                (17 -  X) => (45 - 28+X) 
    if ((ispeed < 3*(ilines/10)) || ((ilines/10) > 9))
    {
      ispeed = 3 * (ilines/10);
    }

    // give them upper number of lines a chance
    if ((ilines/10) > 12)
    {
    	ispeed = 12 + 2 * (ilines/10);
    }
    if ((ilines/10) > 16)
    {
    	ispeed = 28 + (ilines/10);
    }

    // adjust the delays based on the speed of the pieces
    move_rl_delay = 10 - ispeed / 10;
    rotate_delay = 20 - ispeed / 5;

    // calculate score for lines based on number and total lines
    // or the tetris score 
    if (style)
    {
    	iscore += new_lines * new_lines * (int)(500 / (50 - ispeed)) + 1;
    }
    else
    {
      iscore += (scoretable[new_lines])*(ilines/10+1)+1;
    }

    // draw the score and stuff
    draw_score(style, new_lines);
    
    // erase old previews
    for (i = 0; i < 3; i++)
    {
    	draw_piece(13 + Q89_92(4,6)*i, Q89_92(32,31), preview[i], 0);
    }
    
    // update previews and get a random piece
    if (restore)
    {
    	restore = 0;
    }
    else
    {
      // more variables
      x = 4;
      y = 27;
      rotation = 0;

      // cur and preview
      icur = ipreview[0];
      cur = preview[0];
      for (i = 0; i < 2; i++)
      {
        ipreview[i] = ipreview[i+1];
        preview[i] = preview[i+1];
      }

      ipreview[2] = random(7);
      preview[2] = &pieces0[ipreview[2]];
    }

    // draw new previews
    draw_piece(13, Q89_92(32,31), preview[0], 1);
    for (i = 1; i < 3; i++)
    {
    	draw_piece(13 + Q89_92(4,6)*i, Q89_92(32,31), preview[i], 2);
    }

    // update the histogram
    update_block_histogram(block_stats, 6 - cur->id);

    while (moving && !CheckSpecialKeys())
    {
      count++;

      // check for data
      if (getMachine() != NEITHER)
      {
        if (DoMultiplayer(&sent_lines) == FALSE)
        {
        	// according to something in multiplay, we are done
          wins++;
          exit_window("You Win!");
          in_game = FALSE;
        	return result;
        }
      }

      // draw shadow
      if (shadow && (redraw || first_loop))
      {
        first_loop = 0;
        for(i = 0; test_position(x, y + i, cur); i++);
        SetPlane(0);
        draw_piece(x, y+i-1, cur, 2);
      }

      // draw piece
      if (redraw)
      {
        alive = draw_piece(x, y, cur, 1);        
      }

      new_x = x;
      new_y = y;
      new_piece = cur;
      new_rotation = rotation;

      // delay
      delay(1);

      // move down one
      if (count%(50 - ispeed) == 0)
      {
        count = 0;
        can_move_down += (can_move_down == 1);
        can_move_lr += (can_move_lr == 1);
        can_rotate += (can_rotate == 1);
        if (test_position(x, y+1, cur))
        {
          new_y = y + 1;
        }
        else
        {
          // stick it
          alive = draw_piece(x, y, cur, 3);
          moving = 0;
        }
      }

      // moving on down      
      if (can_move_down <= 1)
      {
        // drop key moves the piece as far down as it can go
        if (DropKeyPressed())
        {
          draw_piece(x, y, cur, 0);
          for(i = 0; test_position(x, y + i, cur); i++);
          draw_piece(x, y+i-1, cur, 1);
          alive = draw_piece(x, y+i-1, cur, 3);
          moving = 0;
          can_move_lr++;
          can_rotate++;
          can_move_down += 10000;
        }
        else if (MoveDownPressed())
        {
          // speedy down
          if (test_position(new_x, new_y + 1, new_piece))
          {
            new_y = y + 1;
            can_move_down = 3;
          }
          else
          {
            // if you press down and cannot go down, stick it in place
            alive = draw_piece(x, y, cur, 3);
            moving = 0;
          }
        }
      }
      else
      {
        if (MoveDownPressed() || DropKeyPressed())
        {
          can_move_down--;
        }
        else
        {
          can_move_down = 1;
        }
      }

      // check new_y for clear screen bonus
      if (clear && ((new_y == 35) || (moving == 0)))
      {
        SetPlane(1);
        EraseRect(rect);
        clear = 0;
      }

      // lets get horizontal
      if (can_move_lr <= 1)
      {
        if (MoveRightPressed())
        {
          if (test_position(x+1, new_y, cur))
          {
            new_x = x + 1;
            can_move_lr = first_move_lr * move_rl_delay;
            first_move_lr = 1;
          }
        }

        if (MoveLeftPressed())
        {
          if (test_position(x-1, new_y, cur))
          {
            new_x = x - 1;
            can_move_lr = first_move_lr * move_rl_delay;
            first_move_lr = 1;
          }
        }
      }
      else
      {
        if (MoveRightPressed() || MoveLeftPressed())
        {
          can_move_lr--;
        }
        else
        {
          can_move_lr = 1;
          first_move_lr = 2;
        }
      }

      // rotations 'R' us
      if (can_rotate <= 1)
      {
        if (RotateClockwisePressed() || RotateCounterPressed())
        {
          if (RotateCounterPressed())
          {
            new_rotation = rotation - 1;
            if (new_rotation < 0)
            {
            	new_rotation = cur->max_rotations - 1;
            }
          }
          else
          {
            new_rotation = (rotation+1)%cur->max_rotations;
          }
          
          // try to rotate
          if ((new_rotation == 0) && test_position(new_x, new_y, &pieces0[cur->id]))
          {
            new_piece = &pieces0[cur->id];
          }
          else if ((new_rotation == 1) && test_position(new_x, new_y, &pieces1[cur->id]))
          {
            new_piece = &pieces1[cur->id];
          }
          else if ((new_rotation == 2) && test_position(new_x, new_y, &pieces2[cur->id]))
          {
            new_piece = &pieces2[cur->id];
          }
          else if ((new_rotation == 3) && test_position(new_x, new_y, &pieces3[cur->id]))
          {
            new_piece = &pieces3[cur->id];
          }
          else
          {
            // rotation failed
            new_rotation = rotation;
          }
          
          // if rotated
          if (new_rotation != rotation)
          {
            can_rotate = first_rotate * rotate_delay;
            first_rotate = 1;
          }
        }
      }
      else
      {
        if (RotateClockwisePressed() || RotateCounterPressed())
        {
          can_rotate--;
        }
        else
        {
          can_rotate = 1;
          first_rotate = 2;
        }
      }

      if (EscapeKeyPressed() && getMachine() == NEITHER && CheckSpecialKeys() == FALSE)
      {
        // try to prevent unwanted pauses
        delay(UNWANTED_KEY_PRESS_DELAY);
        quit = 0;

        // clear the prieveiw and board
        for (i = 0; i < 3; i++)
        {
        	draw_piece(13 + Q89_92(4,6)*i, Q89_92(32,31), preview[i], 0);
        }
        SetPlane(0);
        for (i = 0; i < Q89_92(100,127); i++)
        {
          DrawLine(Q89_92(33,46),i,Q89_92(82,105),i,A_REVERSE);
        }
        memset(GetPlane(1),0,LCD_SIZE);

        // draw the quit confirmation dialog
        draw_window(Q89_92(30,70),Q89_92(10,21),Q89_92(130,170),Q89_92(53,66),"Game Paused",1);
        draw_quit(quit);
        ClearAllKeys();

        // get the input, either quit or continue
        turn_off = TURN_OFF_DELAY;
        while (EscapeKeyPressed() == FALSE && EnterKeyPressed() == FALSE)
        {
          turn_off = check_off(turn_off);

          if (UpKeyPressed())
          {
            quit = 0;
            draw_quit(quit);
            turn_off = TURN_OFF_DELAY;
          }
          else if (DownKeyPressed())
          {
            quit = 1;
            draw_quit(quit);
            turn_off = TURN_OFF_DELAY;
          }
        }
        turn_off = TURN_OFF_DELAY;

        // if they pressed Quit
        if ((EnterKeyPressed() && (quit == 1)) || CheckSpecialKeys())
        {
          moving = 0;
          alive = 0;
        }
        else
        {
          ClearAllKeys();

          // draw everything back on the screen
          draw_screen(style);
          draw_piece(13, Q89_92(32,31), preview[0], 1);
          for (i = 1; i < 3; i++)
          {
            draw_piece(13 + Q89_92(4,6)*i, Q89_92(32,31), preview[i], 2);
          }
          draw_score(style, 0);
          draw_board();
          if (shadow == TRUE)
          {
            for(i = 0; test_position(x, y + i, cur); i++);
            SetPlane(0);
            draw_piece(x, y+i-1, cur, 2);
          }
          draw_piece(x, y, cur, 1);        
        }
      }

      // erase shadow
      if (shadow == TRUE && ((x != new_x) || (new_rotation != rotation)))
      {
        for (i = 0; test_position(x, y + i, cur); i++);
        draw_piece(x, y+i-1, cur, 0);
      }
      
      // erase piece
      if ((x != new_x) || (y != new_y) || (new_rotation != rotation))
      {
        draw_piece(x, y, cur, 0);
        x = new_x;
        y = new_y;
        cur = new_piece;
        rotation = new_rotation;
        redraw = 1;
      }
      else
      {
        redraw = 0;
      }
    }
  }

  // RESULTS
  // if they hit quit (.)
  if (CheckSpecialKeys() == TRUE)
  {
    // we quit, so send dead signal winner signal to other calc
    if (getMachine() != NEITHER)
    {
      buffer[0] = 42;
  	  OSWriteLinkBlock(buffer,1);
    }
  }
  else
  {
    // going out of the game WITHOUT restoring.  they lost or quit

    // if they died (not quit)
    if (quit == 0)
    {
      // the other person won
      if (getMachine() != NEITHER)
      {
        buffer[0] = 42;
  	    OSWriteLinkBlock(buffer,1);

        // they lost
        losses++;
      }

      // game over dialog
      exit_window("Game Over!");
    }

    // if single player game, save the score
    if (getMachine() == NEITHER)
    {
    	result = iscore;
    }

    // going out of the game
    in_game = FALSE;
  }

  return result;
}


//////////////////////////////////////////////////////////////
//
// timerFire
//
void timerFire(int *row)
{
  turn_off = check_off(turn_off);

  // Listen for another calculator on the link
  networkListen();
  delay(4);
  if (getMachine() != my_machine)
  {
    my_machine = getMachine();
    if (my_machine == CLIENT)
    {
    	*row = 0;
    }
    draw_menu(*row);
    draw_high_scores();
  }

  // draw the title
  if (title_piece < 34)
  {
    // first part of the q
    if (title_piece == 0)
    {
    	draw_block(1,35,piece_color?239:0);
    }

    // after the q, u, e, and u
    if ((title_piece > 9) && !((title_piece-10)%6))
    {
      col1++;
    }

    // after the q and e
    if (title_piece == 10)
    {
      draw_block(9,33,piece_color?239:0);
    }
    else if (title_piece == 22)
    {
      piece_offset = 12;
      col1 += Q89_92(60,72);
      draw_block(9,33,piece_color?239:0);
    }

    // draw the piece already!
    draw_piece(qx[title_piece-piece_offset],
               qy[title_piece-piece_offset],
               q[title_piece-piece_offset], piece_color);
  }

  if (++title_piece == 34 + piece_color * 16)
  {
    title_piece = 0;
    piece_offset = 0;
    col1 = Q89_92(-9,10);
    piece_color = piece_color?0:1;
  }
}


//////////////////////////////////////////////////////////////
//
// clearBoard()
//
void clearBoard()
{
	int i;
	for (i=0; i<510; board[i++] = 0);
}


//////////////////////////////////////////////////////////////
//
// prepIntro()
//
void prepIntro()
{
  int i;

  FontSetSys(F_4x6);
  for (i = 0; i < 2; i++)
  {
    memset(GetPlane(i),0,LCD_SIZE);
    DrawStr(Q89_92(46,86), Q89_92(46,63), "version: 0.9    by: Kurt Hoffmann", A_NORMAL);
  }
}


//////////////////////////////////////////////////////////////
//
// fancyClear()
//
/*
void fancyClear()
{
  int row = 0;
  BOOL escape = FALSE;

  // have fun clearing the screen
  while (row < Q89_92(50,64) && escape == FALSE)
  {
  	DrawLine(row,0,0,row,A_REVERSE);
  	DrawLine(Q89_92(159,239)-row,0,Q89_92(159,239),row,A_REVERSE);
  	DrawLine(row,Q89_92(99,127),0,Q89_92(99,127)-row,A_REVERSE);
  	DrawLine(Q89_92(159,239)-row,Q89_92(99,127),Q89_92(159,239),Q89_92(99,127)-row,A_REVERSE);
    row++;
  	DrawLine(row,0,0,row,A_NORMAL);
  	DrawLine(Q89_92(159,239)-row,0,Q89_92(159,239),row,A_NORMAL);
  	DrawLine(row,Q89_92(99,127),0,Q89_92(99,127)-row,A_NORMAL);
  	DrawLine(Q89_92(159,239)-row,Q89_92(99,127),Q89_92(159,239),Q89_92(99,127)-row,A_NORMAL);
  	escape = ((_rowread(0) & 0xff) != 0);
  }
  row = 0;
  while (row < Q89_92(30,56) && escape == FALSE)
  {
  	DrawLine(Q89_92(49,63)+row,0,row,Q89_92(49,63),A_REVERSE);
  	DrawLine(Q89_92(109,175)-row,0,Q89_92(159,239)-row,Q89_92(49,63),A_REVERSE);
  	DrawLine(Q89_92(49,63)+row,Q89_92(99,127),row,Q89_92(50,64),A_REVERSE);
  	DrawLine(Q89_92(109,175)-row,Q89_92(99,127),Q89_92(159,239)-row,Q89_92(50,64),A_REVERSE);
    row++;
  	DrawLine(Q89_92(49,63)+row,0,row,Q89_92(49,63),A_NORMAL);
  	DrawLine(Q89_92(109,175)-row,0,Q89_92(159,239)-row,Q89_92(49,63),A_NORMAL);
  	DrawLine(Q89_92(49,63)+row,Q89_92(99,127),row,Q89_92(50,64),A_NORMAL);
  	DrawLine(Q89_92(109,175)-row,Q89_92(99,127),Q89_92(159,239)-row,Q89_92(50,64),A_NORMAL);
  	escape = ((_rowread(0) & 0xff) != 0);
  }
  row = 0;
  while (row < Q89_92(50,64) && escape == FALSE)
  {
  	DrawLine(Q89_92(79,119),row,Q89_92(29,55)+row,Q89_92(49,63),A_REVERSE);
  	DrawLine(Q89_92(79,119),row,Q89_92(129,184)-row,Q89_92(49,63),A_REVERSE);
  	DrawLine(Q89_92(79,119),Q89_92(99,127)-row,Q89_92(29,55)+row,Q89_92(50,64),A_REVERSE);
  	DrawLine(Q89_92(79,119),Q89_92(99,127)-row,Q89_92(129,184)-row,Q89_92(50,64),A_REVERSE);
    row++;
  	DrawLine(Q89_92(79,119),row,Q89_92(29,55)+row,Q89_92(49,63),A_NORMAL);
  	DrawLine(Q89_92(79,119),row,Q89_92(129,184)-row,Q89_92(49,63),A_NORMAL);
  	DrawLine(Q89_92(79,119),Q89_92(99,127)-row,Q89_92(29,55)+row,Q89_92(50,64),A_NORMAL);
  	DrawLine(Q89_92(79,119),Q89_92(99,127)-row,Q89_92(129,184)-row,Q89_92(50,64),A_NORMAL);
  	escape = ((_rowread(0) & 0xff) != 0);
  }
}
*/

//////////////////////////////////////////////////////////////
//
// Main program
//
void _main(void)
{
  int style = 1, shadow = 0, speed = 0, junk = 0;
  unsigned long result = 0;
  int row = 0;
  int optrow = 0;
  INT_HANDLER save_int_1;

  // init variables
  board = malloc(510 * sizeof(int));
  clearBoard();

  // use timer 6 and speed up the clock
  OSFreeTimer(6);
  OSRegisterTimer(6,2);
  pokeIO(0x600017,0xFA);

  // read the values from the file into the variables
  hsc_RW(&style, &shadow, &speed, &junk, FALSE);
  InitKeys();

  // display the proper high score
  if (style == 0)
  {
    current = tetris;
  }

  //fancyClear();

  // turn on grayscale
  save_int_1 = GetIntVec (AUTO_INT_1);
  SetIntVec (AUTO_INT_1, DUMMY_HANDLER);   
  if (GrayMode(GRAY_ON) == FALSE)
  {
  	OSFreeTimer(6);
    return;
  }

  // draw the main screen if we are not restoring
  if (restore == FALSE)
  {
    prepIntro();
    draw_menu(row);
    draw_high_scores();
    my_machine = NEITHER;
    networkPing();
  }

  // initialize randomness
  randomize();

  // clear enter or escape
  while(EnterKeyPressed() == TRUE || EscapeKeyPressed() == TRUE)
  {
    timerFire(&row);
  }

  // the main menus loop until the user presses escape or teacher key
  while (EscapeKeyPressed() == FALSE && CheckSpecialKeys() == FALSE)
  {
    timerFire(&row);

    // if they hit enter and were on the play button
    if ((EnterKeyPressed() || restore) && CheckSpecialKeys() == FALSE)
    {
      // reset the turn off delay
      turn_off = TURN_OFF_DELAY;

      if (row == 0 || restore)
      {
      	// fix col1, because after the main menu we don't move blocks around anymore
        col1 = Q89_92(28,40);

        // play the game
        result = game(style, shadow, speed, junk);

        if (result > current[2].score)
        {
          ClearAllKeys();

          // Black and white mode for high score
          GrayMode(GRAY_OFF);
          SetIntVec (AUTO_INT_1, save_int_1);

          // update high scores
          update_hsc();

          // back to grayscale
          save_int_1 = GetIntVec (AUTO_INT_1);
          SetIntVec (AUTO_INT_1, DUMMY_HANDLER);  
          if (GrayMode(GRAY_ON) == FALSE)
          {
            OSFreeTimer(6);
  	        return;
          }
        }

        // get ready to draw the title screen unless the teacher key was pressed
        if (CheckSpecialKeys() == FALSE && EscapeKeyPressed() == FALSE)
        {
          // reset title variables
          title_piece = 0;
          piece_offset = 0;
          piece_color = 1;
          col1 = Q89_92(-9,10);
          turn_off = TURN_OFF_DELAY;
          clearBoard();

          // intro prepare: clear the screen and say my name
          prepIntro();
          my_machine = NEITHER;
          networkPing();
        }
      }
      else if (row == 1)
      {
        // if they hit enter and were on the options button
        while(EnterKeyPressed() == TRUE || EscapeKeyPressed() == TRUE)
        {
          timerFire(&row);
        }
        char *titles[3] = {"Options", "Options (first calc)"};
        draw_window(Q89_92(2,42),Q89_92(51,69),Q89_92(158,198),Q89_92(99,125),titles[getMachine()],1);

        // menu items
        draw_options(style, shadow, speed, junk, optrow);

        // wait until they hit the escape or enter key
        while (EnterKeyPressed() == FALSE && EscapeKeyPressed() == FALSE)
        {
          timerFire(&row);

          // check up and down key
          if (UpKeyPressed())
          {
            if (--optrow < 0)
            {
            	optrow = 3 - (current == quadra);
            }
            draw_options(style, shadow, speed, junk, optrow);
            while (UpKeyPressed())
            {
            	timerFire(&row);
            }
            turn_off = TURN_OFF_DELAY;
          }
          else if (DownKeyPressed())
          {
            optrow = (optrow + 1) % (4 - (current == quadra));
            draw_options(style, shadow, speed, junk, optrow);
            while (DownKeyPressed())
            {
            	timerFire(&row);
            }
            turn_off = TURN_OFF_DELAY;
          }
          else if (LeftKeyPressed())
          {
            if (optrow == 0)
            {
              style = 0;
              current = tetris;
            }
            else if (optrow == 1)
            {
            	shadow = 0;
            }
            else if (optrow == 2)
            {
            	speed -= speed > 0;
            }
            else if (optrow == 3)
            {
            	junk -= junk > 0;
            }
            draw_options(style, shadow, speed, junk, optrow);
            while (LeftKeyPressed())
            {
            	timerFire(&row);
            }
            turn_off = TURN_OFF_DELAY;
          }
          else if (RightKeyPressed())
          {
            if (optrow == 0)
            {
              style = 1;
              current = quadra;
            }
            else if (optrow == 1)
            {
            	shadow = 1;
            }
            else if (optrow == 2)
            {
            	speed += speed < 9;
            }
            else if (optrow == 3)
            {
            	junk += junk < 9;
            }
            draw_options(style, shadow, speed, junk, optrow);
            while (RightKeyPressed())
            {
            	timerFire(&row);
            }
            turn_off = TURN_OFF_DELAY;
          }      
        }
      }

      // get the title screen back up
      if (CheckSpecialKeys() == FALSE)
      {
        while(EnterKeyPressed() == TRUE || EscapeKeyPressed() == TRUE)
        {
          timerFire(&row);
        }
        draw_menu(row);
        draw_high_scores();
      }
    }
    else if (LeftKeyPressed() && row != 0)
    {
      row = 0;
      menu_items(row);
      while (LeftKeyPressed())
      {
      	timerFire(&row);
      }
      turn_off = TURN_OFF_DELAY;
    }
    else if (RightKeyPressed() && row != 1 && getMachine() != CLIENT)
    {
      row = 1;
      menu_items(row);
      while(RightKeyPressed())
      {
      	timerFire(&row);
      }
      turn_off = TURN_OFF_DELAY;
    }
  }

  // wait until they let up off the enter or escape key
  ClearAllKeys();

  // disable grayscale
  GrayMode(GRAY_OFF);
  SetIntVec (AUTO_INT_1, save_int_1);

  // Restore normal timer speed
  OSFreeTimer(6);
  pokeIO(0x600017,0xB2);

  // write the variables into the file
  hsc_RW(&style, &shadow, &speed, &junk, TRUE);
  SaveKeys();

  // free the board memory
  free(board);

  // key flush
  GKeyFlush();
}
